// This program obtains a number from the user and generates a board with that
// many rows and columns.  The program also obtains a second number from the
// user that specifies how many blanks should be in the board.  The program
// chooses random locations for the blanks and fills the other locations with
// random numbers ranging from 1 to 9 inclusive.
// 
// Aaron Weeden, May 24, 2013.

#include <iostream>
#include <stdlib.h>
#include <time.h>

using namespace std;

const int NON_BLANK = -2; // Represents a spot on the board that doesn't have a
                          // value but also isn't a blank.
const int BLANK = -1; // Represents a blank on the board.

int main() {

    // Declare variables.
    int size; // This number is provided by the user.  The board will have this
              // many rows and columns.
    int numBlanks; // This number is provided by the user.  The board will have
                   // this many blanks in it.
    int **board; // This array contains the value at each position in the
                 // board.
    int rowIdx; // This is used to loop over the rows of the board.
    int colIdx; // This is used to loop over the columns of the board.
    int blankIdx; // This is used to loop over the blanks.

    // Get number from user.
    cout << "Please enter the size of the board (e.g. 9 for a 9x9 board): ";
    cin >> size;

    // Get number of blanks from user.
    cout << "Please enter the number of blanks in the board: ";
    cin >> numBlanks;

    // Make sure the board can fit the number of blanks.  Ask the user to
    // re-input if not.
    while (numBlanks > size * size) {
        cout << "Number of blanks must be less than " << size * size + 1
            << ": ";
        cin >> numBlanks;
    }

    // Initialize the board.
    /// Allocate memory for the board.
    board = (int**)malloc(size * size * sizeof(int));

    /// For each row in the board,
    for (rowIdx = 0; rowIdx < size; rowIdx++) {

        /// Allocate memory for the row.
        board[rowIdx] = (int*)malloc(size * sizeof(int));

        /// For each column,
        for (colIdx = 0; colIdx < size; colIdx++) {

            /// Indicate the position at that row and column is empty, but not
            /// blank.
            board[rowIdx][colIdx] = NON_BLANK;
        }
    }

    // Seed the random number generator.
    srand(time(NULL));

    // Determine the random rows and columns in which each blank will occur.
    /// Generate a random position for the first blank.
    rowIdx = rand() % size;
    colIdx = rand() % size;

    /// For each blank,
    for (blankIdx = 0; blankIdx < numBlanks; blankIdx++) {

        /// While the current position is blank,
        while (board[rowIdx][colIdx] == BLANK) {

            /// Generate a new random position.
            rowIdx = rand() % size;
            colIdx = rand() % size;
        }

        /// Place the blank at the generated random position.
        board[rowIdx][colIdx] = BLANK;
    }

    // For the remaining non-blank positions in the board, generate a random
    // number for each.
    /// For each row in the board,
    for (rowIdx = 0; rowIdx < size; rowIdx++) {

        /// For each column in the board,
        for (colIdx = 0; colIdx < size; colIdx++) {

            /// If the position in the board is non-blank,
            if (board[rowIdx][colIdx] == NON_BLANK) {

                /// Place a random number in the range 1 to 9, inclusive, at 
                /// the position.
                board[rowIdx][colIdx] = rand() % 9 + 1;
            }
        }
    }

    // Print the board.
    /// Print a title.
    cout << "Board: " << endl;

    /// For each row in the board,
    for (rowIdx = 0; rowIdx < size; rowIdx++) {

        /// For each column in the board,
        for (colIdx = 0; colIdx < size; colIdx++) {

            /// If the position is blank,
            if (board[rowIdx][colIdx] == BLANK) {

                /// Print a blank.
                cout << "_ ";

                /// Otherwise,
            } else {

                /// Print the number at the position.
                cout << board[rowIdx][colIdx] << " ";
            }
        }

        // Print a newline in between rows.
        cout << endl;
    }

    // Free allocated memory.
    for (rowIdx = size - 1; rowIdx >= 0; rowIdx--) {
        free(board[rowIdx]);
    }
    free(board);

    return 0;
}
